//
// Dirty test of BLOB support in OGR MySQL driver
//
// The read.cpp program tests data in MySQL 'test' database
// written by write.cpp.
//

#include "ogrsf_frmts.h"
#include <cassert>
#include <cstdio>
#include <fstream>
using namespace std;

int main()
{
    const char *pszDatabase = "MySQL:test,user=mloskot";
    OGRErr err = OGRERR_NONE;

    OGRRegisterAll();

    /////////////////////////////////////////////////////////////////////////
    // Open MySQL database

    OGRDataSource *poDS = NULL;
    poDS = OGRSFDriverRegistrar::Open( pszDatabase, FALSE );
    if( poDS == NULL )
    {
        printf( "Open failed.\n" );
        exit( 1 );
    }

    /////////////////////////////////////////////////////////////////////////
    // Find layer

    OGRLayer *poLayer = NULL;
    poLayer = poDS->GetLayerByName( "t1" );
    if( poDS == NULL )
    {
        printf( "Layer does not exist.\n" );
        exit( 1 );
    }

    /////////////////////////////////////////////////////////////////////////
    // Get the only feature we have added with write.cpp program

    const int fid = 1;
    OGRFeature *poFeature = NULL;
    poFeature = poLayer->GetFeature( fid );
    if( poDS == NULL )
    {
        printf( "Feature with FID=%d does not exist.\n", fid );
        exit( 1 );
    }

    int fdx = poFeature->GetFieldIndex("BINDATA");
    int size = 0;
    GByte* poBin = NULL;
    poBin = poFeature->GetFieldAsBinary(fdx, &size);
    assert(NULL != poBin);

    /////////////////////////////////////////////////////////////////////////
    // Create geometry from WKB stream read from BLOB

    OGRGeometry* poGeom = NULL;
    err = OGRGeometryFactory::createFromWkb( poBin, NULL, &poGeom, size);
    assert(OGRERR_NONE == err);
    assert(NULL != poGeom);

    /////////////////////////////////////////////////////////////////////////
    // Do the same using examplar WKB from the file
    std::ifstream fs("multipoint.wkb", ios::in|ios::binary|ios::ate);
    if (!fs.is_open())
    {
        printf( "Can not open multipoint.wkb file\n" );
        exit( 1 );
    }

    std::ifstream::pos_type size2 = fs.tellg();
    char* buf = new char[size2];
    fs.seekg(0, ios::beg);
    fs.read(buf, size2);
    fs.close();

    OGRGeometry* poGeom2 = NULL;
    err = OGRGeometryFactory::createFromWkb( reinterpret_cast<GByte*>(buf), NULL, &poGeom2, size2);
    assert(OGRERR_NONE == err);
    assert(NULL != poGeom2);

    /////////////////////////////////////////////////////////////////////////
    // Print both geometries as a test

    char* wkt1 = NULL;
    err = poGeom->exportToWkt(&wkt1);
    assert(OGRERR_NONE == err);

    char* wkt2 = NULL;
    err = poGeom2->exportToWkt(&wkt2);
    assert(OGRERR_NONE == err);

    printf("Geometry read from BLOB field: \n\n %s \n\n", wkt1);
    printf("Geometry read from file: \n\n %s \n\n", wkt2);

    OGRFree( wkt1 );
    OGRFree( wkt2 );
    OGRGeometryFactory::destroyGeometry( poGeom );
    OGRFeature::DestroyFeature( poFeature );
    OGRDataSource::DestroyDataSource( poDS );
}
